#include "alias.h"

#include "../algorithms.h"

#include <fstream>
using namespace std;

//////// disk access ////////

disk_name		diskname;
alias_data		diskdata;
info_snapshot	readdata;

bool save_aliases()
{
	ofstream file("alias.dat", ios::binary);
	if (!file) return false;

	_listnode <alias_name> *parse = AliasList.head;

	while (parse)
	{
		int i;

		alias_name *alias = parse->item;

		int count = 0;
		for (i = 0; i < MEMORY_LENGTH; ++i)
		{
			alias_data *data = alias->data[i];
			if (!data) break;
			++count;
		}

		strncpy(diskname.name, alias->name, 24);
		diskname.seen = alias->seen;
		diskname.timediff = alias->timediff;
		diskname.data_count = count;

		if (!file.write((char*)&diskname, sizeof(diskname)))
			return false;

		for (i = 0; i < count; ++i)
		{
			alias_data *data = alias->data[i];

			if (!file.write((char*)data, sizeof(alias_data)))
				return false;
		}

		parse = parse->next;
	}

	return true;
}

bool load_aliases()
{
	ifstream file("alias.dat", ios::binary);
	if (!file) return false;

    while (file.read((char*)&diskname, sizeof(diskname)))
	{
		alias_name *name = new alias_name;

		strncpy(name->name, diskname.name, 24);
		name->name_hash = get_name_hash(diskname.name);
		name->seen = diskname.seen;
		name->timediff = diskname.timediff;
		name->data[0] = NULL;

		AliasList.append(name);

		for (int i = 0; i < diskname.data_count; ++i)
		{
			if (!file.read((char*)&diskdata, sizeof(diskdata)))
				return false;

			readdata.id = diskdata.id;
			readdata.ip = diskdata.ip;
			readdata.tzb = diskdata.tzb;

			AddData(name, readdata);
		}
	}

	return true;
}


//////// utils ////////

Uint32 get_name_hash(const char *name)
{
	Uint32 b, hash = 0;

	while (b = *name++)
	{
		hash = ROR(hash, 5) + b;
	}

	return hash;
}

Uint32 get_ip(const char *cp)
{
	Uint32 ip = 0;

	char c, buffer[32];
	int i = 0;

	do
	{
		c = *cp++;

		if ((c == '.') || (c == '\0'))
		{
			buffer[i] = '\0';
			i = 0;

			int node = getInteger(buffer, 10);

			ip >>= 8;
			ip |= (node & 255) << 24;
		}
		else
			buffer[i++] = c;
	} while (c);

	return ip;
}


//////// Zip list ////////

_linkedlist <zip_info> ZipList;

void KillZip(zip_info *zip)
{
	ZipList.kill(zip);
}

int ZipElapsed(zip_info *zip)
{
	time_t now;
	time(&now);

	return int(now - zip->creation);
}

void ResetZipTimer(zip_info *zip)
{
	time_t now;
	time(&now);

	zip->creation = now;
}

zip_info *AddZip(alias_name *alias, int minutes)
{
	zip_info *zip = new zip_info;
	zip->alias = alias;
	zip->minutes = minutes;
	time(&zip->creation);
	ZipList.append(zip);

	return zip;
}

zip_info *MatchZip(alias_name *alias)
{
	_listnode <zip_info> *parse = ZipList.head;

	while (parse)
	{
		zip_info *zip = parse->item;

		alias_name *palias = zip->alias;

		if (palias == alias)
			return zip;

		for (int i = 0; i < MEMORY_LENGTH; ++i)
		{
			alias_data *pdata = palias->data[i];
			if (!pdata) break;

			for (int j = 0; j < MEMORY_LENGTH; ++j)
			{
				alias_data *data = alias->data[j];
				if (!data) break;

				if ((data->id == pdata->id) && (pdata->id != 0))
				{
					return zip;
				}
				else if (data->ip == pdata->ip)
				{
					return zip;
				}
			}
		}

		if (palias->timediff != 0)
		{
			Sint32 timediff = alias->timediff - palias->timediff;
			if (timediff < 0) timediff = -timediff;

			if (timediff <= 150)
			{
				return zip;
			}
		}

		parse = parse->next;
	}

	return NULL;
}

zip_info *FindZip(char *name)
{
	_listnode <zip_info> *parse = ZipList.head;

	tolower(name);
	Uint32 hash = get_name_hash(name);

	while (parse)
	{
		zip_info *zip = parse->item;
		alias_name *alias = zip->alias;

		if (hash == alias->name_hash)
		if (CMPSTR(name, alias->name))
			return zip;

		parse = parse->next;
	}

	return NULL;
}


//////// Alias list ////////

_linkedlist <alias_name> AliasList;

bool ListChanged = false;

alias_name *AddAlias(info_snapshot &snapshot)
{
	alias_name *name = new alias_name;

	strncpy(name->name, snapshot.name, 24);
	name->data[0] = NULL;

	AliasList.append(name);

	return name;
}

bool old_info(alias_name *name, info_snapshot &snapshot)
{
	bool ip_match = false;
	bool id_match = false;

	for (int i = 0; i < MEMORY_LENGTH; ++i)
	{
		alias_data *data = name->data[i];
		if (!data) break;

		if ((snapshot.ip & 0x0000ffff) == (data->ip & 0x0000ffff))
		{
			ip_match = true;
		}

		if (snapshot.id == data->id)
		{
			id_match = true;
		}
	}

	return (ip_match && id_match);
}

void AddData(alias_name *name, info_snapshot &snapshot)
{
	for (int i = 0; i < MEMORY_LENGTH; ++i)
	{
		if (!name->data[i]) break;
	}

	alias_data *data = new alias_data;

	data->id = snapshot.id;
	data->ip = snapshot.ip;
	data->tzb = snapshot.tzb;
	name->timediff = snapshot.timediff;

	if (i == MEMORY_LENGTH)
	{
		// no more room
		for (int i = 3; i < MEMORY_LENGTH - 1; ++i)
		{
			name->data[i] = name->data[i + 1];
		}

		name->data[MEMORY_LENGTH - 1] = data;
	}
	else
	{
		// append

		name->data[i++] = data;

		if (i != MEMORY_LENGTH)
			name->data[i] = NULL;
	}

	ListChanged = true;
}

void UpdateHash(alias_name &snapshot)
{
	tolower(snapshot.name);
	snapshot.name_hash = get_name_hash(snapshot.name);
}

void UpdateSeen(alias_name &snapshot)
{
	time(&snapshot.seen);
}

alias_name *FindSnapshot(char *name)
{
	_listnode <alias_name> *parse = AliasList.head;

	char tName[24];
	memcpy(tName, name, 24);
	tolower(tName);
	Uint32 hash = get_name_hash(tName);

	while (parse)
	{
		alias_name *alias = parse->item;

		if (hash == alias->name_hash)
		if (CMPSTR(tName, alias->name))
			return alias;

		parse = parse->next;
	}

	return NULL;
}

String GetLastSeen(char *name)
{
	alias_name *alias = FindSnapshot(name);

	if (alias)
	{
		return asctime(gmtime(&alias->seen));
	}
	else
	{
		return "Haven't seen them.";
	}
}


//////// *tinfo parser ////////

bool parse_tinfo(char *msg, info_snapshot &snapshot)
{
	int len = (int)strlen(msg);

	int i, buff_offset;
	char c, buffer[32];
	bool started;

	/*
			server		user		diff
		" ##########  ##########  ##########"

		#'s are padded on the left with spaces
	*/

	if (len != 35) return false;

	/* server time */

	if (msg[0] != ' ') return false;

	started = false;

	for (i = 1; i <= 10; ++i)
	{
		c = msg[i];

		if (c == ' ')
		{
			if (started)
			{
				return false;
			}
		}
		else if ( ('0' <= c) && (c <= '9') )
		{
			started = true;
		}
		else
		{
			return false;
		}
	}

	/* user time */

	if (msg[11] != ' ') return false;
	if (msg[12] != ' ') return false;

	started = false;

	for (i = 13; i <= 22; ++i)
	{
		c = msg[i];

		if (c == ' ')
		{
			if (started)
			{
				return false;
			}
		}
		else if ( ('0' <= c) && (c <= '9') )
		{
			started = true;
		}
		else
		{
			return false;
		}
	}

	/* diff time */

	if (msg[23] != ' ') return false;
	if (msg[24] != ' ') return false;

	started = false;
	buff_offset = 0;

	for (i = 25; i <= 34; ++i)
	{
		c = msg[i];

		if (c == ' ')
		{
			if (started)
			{
				return false;
			}
		}
		else if ( ( ('0' <= c) && (c <= '9') ) || (c == '-') )
		{
			buffer[buff_offset++] = c;
			started = true;
		}
		else
		{
			return false;
		}
	}

	buffer[buff_offset] = '\0';
	snapshot.timediff = getInteger(buffer, 10);

	/* Done! */

	return true;
}


//////// *info parser ////////

bool parse_info(char *msg, info_snapshot &snapshot)
{
	char c, buffer[64];
	int buff_offset;

	/*
		"IP:111.111.111.111 "
	*/

	/* Align IP string */

	if ((*msg++ != 'I') ||
		(*msg++ != 'P') ||
		(*msg++ != ':')  )
		return false;

	/* Read IP string */

	buff_offset = 0;

	while (c = *msg++)
	{
		if (c == ' ')
		{
			if (buff_offset == 0) return false;

			buffer[buff_offset] = '\0';

			int ip = get_ip(buffer);
			if (!ip) return false;
			snapshot.ip = ip;

			break;
		}
		else if ( (('0' <= c) && (c <= '9')) || (c == '.') )
		{
			buffer[buff_offset++] = c;
			if (buff_offset >= 16) return false;
		}
		else
		{
			return false;
		}
	}

	if (!c) return false;

	/*
		" TimeZoneBias:111 "
	*/

	/* Align TimeZoneBias string */

	if ((*msg++ != ' ') ||
		(*msg++ != 'T') ||
		(*msg++ != 'i') ||
		(*msg++ != 'm') ||
		(*msg++ != 'e') ||
		(*msg++ != 'Z') ||
		(*msg++ != 'o') ||
		(*msg++ != 'n') ||
		(*msg++ != 'e') ||
		(*msg++ != 'B') ||
		(*msg++ != 'i') ||
		(*msg++ != 'a') ||
		(*msg++ != 's') ||
		(*msg++ != ':')  )
		return false;

	/* Read TimeZoneBias string */

	buff_offset = 0;

	while (c = *msg++)
	{
		if (c == ' ')
		{
			if (buff_offset == 0) return false;

			buffer[buff_offset] = '\0';

			snapshot.tzb = getInteger(buffer, 10);

			break;
		}
		else if ( (('0' <= c) && (c <= '9')) || (c == '-') )
		{
			buffer[buff_offset++] = c;
			if (buff_offset >= 7) return false;
		}
		else
		{
			return false;
		}
	}

	if (!c) return false;

	/*
		" Freq:1111 "
	*/

	/* Align Freq string */

	if ((*msg++ != ' ') ||
		(*msg++ != 'F') ||
		(*msg++ != 'r') ||
		(*msg++ != 'e') ||
		(*msg++ != 'q') ||
		(*msg++ != ':')  )
		return false;

	/* Skip Freq string */

	while (c = *msg++)
		if (c == ' ')
			break;

	if (!c) return false;

	/*
		" TypedName:aaaa "
	*/

	/* Align TypedName string */

	if ((*msg++ != ' ') ||
		(*msg++ != 'T') ||
		(*msg++ != 'y') ||
		(*msg++ != 'p') ||
		(*msg++ != 'e') ||
		(*msg++ != 'd') ||
		(*msg++ != 'N') ||
		(*msg++ != 'a') ||
		(*msg++ != 'm') ||
		(*msg++ != 'e') ||
		(*msg++ != ':')  )
		return false;

	char *name = msg;

	/*
		Here's the hard part.  The TypedName: field could contain some of our parse strings.
		Let's find out the MacId first, then go backwards and meet in the middle

		"TypedName:aaa  Demo:N  MachineId:11111111"
	*/

	/* Read MachineId string */

	while (*msg++);
	msg -= 2;
	while (*msg-- != ':');
	msg += 2;

	snapshot.id = getInteger(msg, 10);

	char *ename = msg - 20;
	msg = ename;

	/* Validate end of string */

	if ((*msg++ != ' ') ||
		(*msg++ != ' ') ||
		(*msg++ != 'D') ||
		(*msg++ != 'e') ||
		(*msg++ != 'm') ||
		(*msg++ != 'o') ||
		(*msg++ != ':')  )
		return false;

	c = *msg++;
	if ( (c != '0') && (c != '1') )
		return false;

	if ((*msg++ != ' ') ||
		(*msg++ != ' ') ||
		(*msg++ != 'M') ||
		(*msg++ != 'a') ||
		(*msg++ != 'c') ||
		(*msg++ != 'h') ||
		(*msg++ != 'i') ||
		(*msg++ != 'n') ||
		(*msg++ != 'e') ||
		(*msg++ != 'I') ||
		(*msg++ != 'd') ||
		(*msg++ != ':')  )
		return false;

	/* Read TypedName string */

	buff_offset = 0;

	while (name != ename)
	{
		snapshot.name[buff_offset++] = *name++;
		if (buff_offset >= 24) return false;
	}

	snapshot.name[buff_offset] = '\0';

	/* Done! */

	return true;
}
