/*
*/

#pragma once
#ifndef ALIAS_H
#define ALIAS_H

#define STRING_CAST_CHAR
#include "../datatypes.h"

#include <time.h>

#define HTONL(x) ( (((x) & 0x000000ff) << 24) | (((x) & 0x0000ff00) << 8) | (((x) & 0x00ff0000) >> 8) | (((x) & 0xff000000) >> 24) )

Uint32 get_ip(const char *cp);
Uint32 get_name_hash(const char *name);


/* *info & *tinfo Snapshot */
struct info_snapshot
{
	Uint32 name_hash;
	char name[24];
	Uint32 ip;
	Sint32 tzb;
	Sint32 id;
	Sint32 timediff;
	time_t seen;
};


/* Efficient memory storage */
#define MEMORY_LENGTH 10

struct alias_data
{
	Uint32 ip;
	Sint32 tzb;
	Sint32 id;
};

struct alias_name
{
	Uint32 name_hash;
	char name[24];
	time_t seen;
	Sint32 timediff;

	alias_data *data[MEMORY_LENGTH];
};

extern _linkedlist <alias_name> AliasList;
extern bool ListChanged;

struct zip_info
{
	alias_name *alias;

	time_t creation;
	Uint32 minutes;

	char creator[20];
	Sint32 access;

	bool b, s, c;
};

extern _linkedlist <zip_info> ZipList;


/* Disk access */

struct disk_name
{
	char name[24];
	time_t seen;
	Sint32 timediff;
	Sint32 data_count;
};

bool save_aliases();
bool load_aliases();


/* Accessors */

void UpdateHash(alias_name &snapshot);
void UpdateSeen(alias_name &snapshot);

alias_name *AddAlias(info_snapshot &snapshot);
void AddData(alias_name *name, info_snapshot &snapshot);
alias_name *FindSnapshot(char *name);
bool old_info(alias_name *name, info_snapshot &snapshot);
String GetLastSeen(char *name);

zip_info *FindZip(char *name);
zip_info *MatchZip(alias_name *alias);
zip_info *AddZip(alias_name *alias, int minutes);
void KillZip(zip_info *zip);
int ZipElapsed(zip_info *zip);
void ResetZipTimer(zip_info *zip);


/* Parsers */

bool parse_tinfo(char *msg, info_snapshot &snapshot);
bool parse_info(char *msg, info_snapshot &snapshot);

#endif	// ALIAS_H
