#include "spawn.h"

#include <string.h>
#include "..\algorithms.h"

Uint32 node_mask[4] = {0x00000000, 0x000000ff, 0x0000ffff, 0x00ffffff};

void botInfo::gotHelp(Player *p, Command *c)
{
	// List commands

	if (!*c->final)
	{
		switch (p->access)
		{
		case OP_Duke:
		case OP_Baron:
		case OP_King:
		case OP_Emperor:
		case OP_RockStar:
		case OP_Q:
		case OP_God:
		case OP_Owner:
			{	// Owner-level commands
			}
		case OP_SysOp:
			{	// SysOp-level commands
			}
		case OP_SuperModerator:
			{	// SuperModerator-level commands
				sendPrivate(p, "Ext: !version !status !about !seen !findid !findip !findtime !alias !whois");
				sendPrivate(p, "Ext: !ban !speclock !silence !zip !unzip !listzip !zipchan");
			}
			break;
		case OP_Moderator:
			{	// Moderator-level commands
				sendPrivate(p, "Ext: !status !about !seen !alias");
				sendPrivate(p, "Ext: !ban !speclock !silence !zip !unzip !listzip !zipchan");
			}
			break;
		case OP_Limited:
			{	// Limited-level commands
				sendPrivate(p, "Ext: !status !about !seen");
			}
			break;
		case OP_Player:
			{	// Player-level commands
				sendPrivate(p, "!about (query me about my function)");
				sendPrivate(p, "!seen <name> (when did you see them last?)");
			}
		}

		return;
	}

	// Specific command help

	switch (p->access)
	{
	case OP_Duke:
	case OP_Baron:
	case OP_King:
	case OP_Emperor:
	case OP_RockStar:
	case OP_Q:
	case OP_God:
	case OP_Owner:
		{	// Owner-level commands
			if (c->checkParam("owner") || c->checkParam("all"))
			{
				sendPrivate(p, "Ext: !version !status !about !seen !findid !findip !findtime !alias !whois");
				sendPrivate(p, "Ext: !ban !speclock !silence !zip !unzip !listzip !zipchan");
			}
		}
	case OP_SysOp:
		{	// SysOp-level commands
			if (c->checkParam("sop") || c->checkParam("all"))
			{
				sendPrivate(p, "Ext: !version !status !about !seen !findid !findip !findtime !alias !whois");
				sendPrivate(p, "Ext: !ban !speclock !silence !zip !unzip !listzip !zipchan");
			}
			else if (c->checkParam("findip"))
			{
				sendPrivate(p, "!findip <127.0.0.1> (look for players with matching ip's)");
				sendPrivate(p, "!findip <127.0.*>");
				sendPrivate(p, "!findip <127.0.0.3-127.0.0.8>");
			}
		}
	case OP_SuperModerator:
		{	// SuperModerator-level commands
			if (c->checkParam("smod") || c->checkParam("all"))
			{
				sendPrivate(p, "Ext: !version !status !about !seen !findid !findip !findtime !alias !whois");
				sendPrivate(p, "Ext: !ban !speclock !silence !zip !unzip !listzip !zipchan");
			}
			else if (c->checkParam("findid"))
			{
				sendPrivate(p, "!findid <machine id> (look for players with matching machine id's)");
			}
			else if (c->checkParam("findtime"))
			{
				sendPrivate(p, "!findtime <delta time> (look for players with similar time differences)");
			}
		}
	case OP_Moderator:
		{	// Moderator-level commands
			if (c->checkParam("mod") || c->checkParam("all"))
			{
				sendPrivate(p, "Ext: !status !about !seen !alias");
				sendPrivate(p, "Ext: !ban !speclock !silence !zip !unzip !listzip !zipchan");
			}
			else if (c->checkParam("alias"))
			{
				sendPrivate(p, "!alias [-a] <player name> (find aliases for given player name)");
				sendPrivate(p, "!alias -a                 (include approximate IP matches: ~IP ~~IP)");
			}
			else if (c->checkParam("ban"))
			{
				sendPrivate(p, "!ban [-t=<minutes>] <player name> (shortcut for !zip -b [-t=time] <name>)");
			}
			else if (c->checkParam("speclock"))
			{
				sendPrivate(p, "!speclock [-t=<minutes>] <player name> (shortcut for !zip -c [-t=time] <name>)");
			}
			else if (c->checkParam("silence"))
			{
				sendPrivate(p, "!silence [-t=<minutes>] <player name> (shortcut for !zip -s [-t=time] <name>)");
			}
			else if (c->checkParam("zip"))
			{
				sendPrivate(p, "!zip [-b] [-s] [-c] [-t=<minutes>] <player name> (create or change a zipped player)");
				sendPrivate(p, "!zip -b           (auto-*kill zipped player)");
				sendPrivate(p, "!zip -s           (auto-*shutup zipped player)");
				sendPrivate(p, "!zip -c           (auto-*spec zipped player)");
				sendPrivate(p, "!zip -t=<minutes> (by default, effect lasts for 10 minutes)");
			}
			else if (c->checkParam("unzip"))
			{
				sendPrivate(p, "!unzip <player name> (remove a player zip)");
			}
			else if (c->checkParam("listzip"))
			{
				sendPrivate(p, "!listzip -bsc <player name> (list zipped players using the tags)");
			}
			else if (c->checkParam("zipchan"))
			{
				sendPrivate(p, "!zipchan [on/off] (log zip-events to channel 1?)");
			}
		}
	case OP_Limited:
		{	// Limited-level commands
			if (c->checkParam("status"))
			{
				sendPrivate(p, "!status (how're ya doing bot?)");
			}
		}
	case OP_Player:
		{	// Player-level commands
			if (c->checkParam("about"))
			{
				sendPrivate(p, "!about (query me about my function)");
			}
			else if (c->checkParam("seen"))
			{
				sendPrivate(p, "!seen <name> (when did you see them last?)");
			}
		}
	}
}

void botInfo::gotCommand(Player *p, Command *c)
{
	if (!p) return;
	if (!c) return;

	switch (p->access)
	{
	case OP_Duke:
	case OP_Baron:
	case OP_King:
	case OP_Emperor:
	case OP_RockStar:
	case OP_Q:
	case OP_God:
	case OP_Owner:
		{	// Owner-level commands
		}
	case OP_SysOp:
		{	// SysOp-level commands
		}
	case OP_SuperModerator:
		{	// SuperModerator-level commands
			if (c->check("version"))
			{
				sendPrivate(p, "[name:alias.dll] [maker:cat02e@fsu.edu] [build:10]");
			}
			else if (c->check("whois"))
			{
				alias_name *alias = FindSnapshot(c->final);

				if (alias)
				{
					String s;
					s += "Name:";
					s += alias->name;
					s += "  DELT:";
					s += alias->timediff;
					sendPrivate(p, s);

					for (int i = 0; i < MEMORY_LENGTH; ++i)
					{
						alias_data *data = alias->data[i];
						if (!data) break;

						s = "MacId:";
						s += data->id;
						s += "  IP:";
						s += int(((BYTE*)&data->ip)[0]);
						s += ".";
						s += int(((BYTE*)&data->ip)[1]);
						s += ".";
						s += int(((BYTE*)&data->ip)[2]);
						s += ".";
						s += int(((BYTE*)&data->ip)[3]);
						s += "  TZB:";
						s += data->tzb;
						sendPrivate(p, s);
					}
				}
				else
				{
					sendPrivate(p, "Haven't seen them.");
				}

				break;
			}
			else if (c->check("findip"))
			{
				/*
					127.0.0.1
					127.0.*
					127.0.0.0-127.0.0.255
				*/

				Uint32 ip1, ip2;

				String addr2 = c->final;
				String addr1 = addr2.split('-');

				bool ranged = false;
				if (addr1 != addr2)
					ranged = true;

				if (ranged)
				{
					ip1 = get_ip(addr1);
					ip2 = get_ip(addr2);
				}
				else
				{
					String addr[4];

					addr[0] = addr2.split('.');
					addr[1] = addr2.split('.');
					addr[2] = addr2.split('.');
					addr[3] = addr2;

					int node_offset;

					for (int i = 0; i < 4; ++i)
					{
						if (addr[i] == "*")
						{
							ranged = true;
							node_offset = i;
							break;
						}
					}

					if (ranged)
					{
						int i;
						String ip;

						for (i = 0; i < node_offset; ++i)
						{
							ip += addr[i];
							ip += ".";
						}

						for (i = node_offset; i < 3; ++i)
						{
							ip += "0.";
						}

						ip += "0";

						ip1 = get_ip(ip);

						ip1 &= node_mask[node_offset];

						ip2 = ip1;

						ip2 |= ~node_mask[node_offset];
					}
					else
					{
						ip1 = get_ip(addr1);
						ip2 = ip1;
					}
				}

				ip1 = HTONL(ip1);
				ip2 = HTONL(ip2);

				bool found = false;
				int count = 0, tcount = 0;
				String s = "Alias";
				_listnode <alias_name> *parse = AliasList.head;

				while (parse)
				{
					alias_name *alias = parse->item;

					for (int i = 0; i < MEMORY_LENGTH; ++i)
					{
						alias_data *data = alias->data[i];
						if (!data) break;

						Uint32 ip = HTONL(data->ip);

						if ( (ip1 <= ip) && (ip <= ip2) )
						{
							found = true;
							s += " :";
							s += alias->name;
							s += ":";
							++count;

							if (count == 10)
							{
								sendPrivate(p, s);
								s = "Alias";
								count = 0;
							}

							if (p->access < OP_SysOp)
							if (++tcount >= 10)
							{
								sendPrivate(p, "Cut list off short due to your access level.");
								break;
							}

							break;
						}
					}

					parse = parse->next;
				}

				if (!found) sendPrivate(p, "No matches.");
				else if (count != 0) sendPrivate(p, s);
			}
			else if (c->check("findid"))
			{
				if (!isNumeric(c->final))
				{
					sendPrivate(p, "Syntax: !findid <machine id>");
					break;
				}

				Sint32 id = getInteger(c->final, 10);

				bool found = false;
				int count = 0, tcount = 0;
				String s = "Alias";
				_listnode <alias_name> *parse = AliasList.head;

				while (parse)
				{
					alias_name *alias = parse->item;

					for (int i = 0; i < MEMORY_LENGTH; ++i)
					{
						alias_data *data = alias->data[i];
						if (!data) break;

						if (data->id == id)
						{
							found = true;
							s += " :";
							s += alias->name;
							s += ":";
							++count;

							if (count == 10)
							{
								sendPrivate(p, s);
								s = "Alias";
								count = 0;
							}

							if (p->access < OP_SysOp)
							if (++tcount >= 10)
							{
								sendPrivate(p, "Cut list off short due to your access level.");
								break;
							}

							break;
						}
					}

					parse = parse->next;
				}

				if (!found) sendPrivate(p, "No matches.");
				else if (count != 0) sendPrivate(p, s);
			}
			else if (c->check("findtime"))
			{
				if (!isNumeric(c->final))
				{
					sendPrivate(p, "Syntax: !findtime <delta time>");
					break;
				}

				Sint32 dtime = getInteger(c->final, 10);

				bool found = false;
				int count = 0, tcount = 0;
				String s = "Alias";
				_listnode <alias_name> *parse = AliasList.head;

				time_t current;
				time(&current);

				while (parse)
				{
					alias_name *alias = parse->item;

					Sint32 tdiff = alias->timediff - dtime;

					if (tdiff < 0) tdiff = -tdiff;

					if (tdiff <= 150)
					{
						found = true;
						s += " :";
						s += alias->name;
						s += "(";
						s += tdiff;
						s += "; ";
						s += Sint32(current - alias->seen) / 60;
						s += " min. ago):";
						++count;

						if (count == 10)
						{
							sendPrivate(p, s);
							s = "Alias";
							count = 0;

							if (p->access < OP_SysOp)
							if (++tcount >= 10)
							{
								sendPrivate(p, "Cut list off short due to your access level.");
								break;
							}
						}
					}

					parse = parse->next;
				}

				if (!found) sendPrivate(p, "No matches.");
				else if (count != 0) sendPrivate(p, s);
			}
		}
	case OP_Moderator:
		{	// Moderator-level commands
			if (c->check("zipchan"))
			{
				if (c->checkParam("on"))
				{
					send_to_channel = true;
				}
				else if (c->checkParam("off"))
				{
					send_to_channel = false;
				}

				if (send_to_channel)
					sendPrivate(p, "Sending zip-events to channel 1.");
				else
					sendPrivate(p, "Not sending zip-events to channel 1.");
			}
			else if (c->check("ban"))
			{
				_switch *sw = c->getParam('t');
				int time	= ((sw != NULL) ? getInteger(sw->param, 10) : 10);

				// Validate parameters
				if (time < 1)
				{
					sendPrivate(p, "You must specify a time higher than 0: !ban -t=5 <name>");

					break;
				}

				// Find the alias snapshot of the name given
				alias_name *alias = FindSnapshot(c->final);

				if (!alias)
				{
					sendPrivate(p, "I have no record of the name you gave me.");

					break;
				}

				// Find existing zip entry
				zip_info *zip = FindZip(c->final);

				if (!zip)
				{	// Create a new zip if none was found
					zip = AddZip(alias, time);
				}
				else
				{
					if (p->access < zip->access)
					{
						sendPrivate(p, "You may not modify a zip made by a higher-level operator.");

						break;
					}

					if (sw)
					{
						zip->minutes = time;
						ResetZipTimer(zip);
					}
				}

				zip->b = true;
				zip->s = false;
				zip->c = false;

				memcpy(zip->creator, p->name, 20);
				zip->creator[13] = '\0';
				zip->access = p->access;

				// Return OK-string
				String s;
				s += "Successfully zipped " + String(c->final) + ".  +ban  Timer: ";
				s += zip->minutes;
				s += "  Access level: ";
				s += zip->access;

				sendPrivate(p, s);

				// Apply settings to player in the zone
				_listnode <Player> *parse = playerlist->head;

				while (parse)
				{
					Player *pp = parse->item;

					if (CMPSTR(pp->name, c->final))
					{
						CheckZip(pp, alias);

						break;
					}

					parse = parse->next;
				}
			}
			else if (c->check("speclock"))
			{
				_switch *sw = c->getParam('t');
				int time	= ((sw != NULL) ? getInteger(sw->param, 10) : 10);

				// Validate parameters
				if (time < 1)
				{
					sendPrivate(p, "You must specify a time higher than 0: !speclock -t=5 <name>");

					break;
				}

				// Find the alias snapshot of the name given
				alias_name *alias = FindSnapshot(c->final);

				if (!alias)
				{
					sendPrivate(p, "I have no record of the name you gave me.");

					break;
				}

				// Find existing zip entry
				zip_info *zip = FindZip(c->final);

				if (!zip)
				{	// Create a new zip if none was found
					zip = AddZip(alias, time);
				}
				else
				{
					if (p->access < zip->access)
					{
						sendPrivate(p, "You may not modify a zip made by a higher-level operator.");

						break;
					}

					if (sw)
					{
						zip->minutes = time;
						ResetZipTimer(zip);
					}
				}

				zip->b = false;
				zip->s = false;
				zip->c = true;

				memcpy(zip->creator, p->name, 20);
				zip->creator[13] = '\0';
				zip->access = p->access;

				// Return OK-string
				String s;
				s += "Successfully zipped " + String(c->final) + ".  +spec  Timer: ";
				s += zip->minutes;
				s += "  Access level: ";
				s += zip->access;

				sendPrivate(p, s);

				// Apply settings to player in the zone
				_listnode <Player> *parse = playerlist->head;

				while (parse)
				{
					Player *pp = parse->item;

					if (CMPSTR(pp->name, c->final))
					{
						CheckZip(pp, alias);

						break;
					}

					parse = parse->next;
				}
			}
			else if (c->check("silence"))
			{
				_switch *sw = c->getParam('t');
				int time	= ((sw != NULL) ? getInteger(sw->param, 10) : 10);

				// Validate parameters
				if (time < 1)
				{
					sendPrivate(p, "You must specify a time higher than 0: !silence -t=5 <name>");

					break;
				}

				// Find the alias snapshot of the name given
				alias_name *alias = FindSnapshot(c->final);

				if (!alias)
				{
					sendPrivate(p, "I have no record of the name you gave me.");

					break;
				}

				// Find existing zip entry
				zip_info *zip = FindZip(c->final);

				if (!zip)
				{	// Create a new zip if none was found
					zip = AddZip(alias, time);
				}
				else
				{
					if (p->access < zip->access)
					{
						sendPrivate(p, "You may not modify a zip made by a higher-level operator.");

						break;
					}

					if (sw)
					{
						zip->minutes = time;
						ResetZipTimer(zip);
					}
				}

				zip->b = false;
				zip->s = true;
				zip->c = false;

				memcpy(zip->creator, p->name, 20);
				zip->creator[13] = '\0';
				zip->access = p->access;

				// Return OK-string
				String s;
				s += "Successfully zipped " + String(c->final) + ".  +silence  Timer: ";
				s += zip->minutes;
				s += "  Access level: ";
				s += zip->access;

				sendPrivate(p, s);

				// Apply settings to player in the zone
				_listnode <Player> *parse = playerlist->head;

				while (parse)
				{
					Player *pp = parse->item;

					if (CMPSTR(pp->name, c->final))
					{
						CheckZip(pp, alias);

						break;
					}

					parse = parse->next;
				}
			}
			else if (c->check("zip"))
			{
				bool ban	= (c->getParam('b') != NULL);
				bool shutup	= (c->getParam('s') != NULL);
				bool spec	= (c->getParam('c') != NULL);

				if (ban) { shutup = false; spec = false; }

				_switch *sw = c->getParam('t');
				int time	= ((sw != NULL) ? getInteger(sw->param, 10) : 10);

				// Validate parameters
				if (time < 1)
				{
					sendPrivate(p, "You must specify a time higher than 0: !zip -bsct=5 <name>");

					break;
				}

				// Find the alias snapshot of the name given
				alias_name *alias = FindSnapshot(c->final);

				if (!alias)
				{
					sendPrivate(p, "I have no record of the name you gave me.");

					break;
				}

				// Find existing zip entry
				zip_info *zip = FindZip(c->final);

				if (!zip)
				{	// Create a new zip if none was found
					if (!ban && !shutup && !spec)
					{
						sendPrivate(p, "You must specify at least one tag: !zip -bsc -t=5 <name>");

						break;
					}

					zip = AddZip(alias, time);
				}
				else
				{
					if (p->access < zip->access)
					{
						sendPrivate(p, "You may not modify a zip made by a higher-level operator.");

						break;
					}

					if (sw)
					{
						zip->minutes = time;
						ResetZipTimer(zip);
					}
				}

				if (ban || shutup || spec)
				{
					zip->b = ban;
					zip->s = shutup;
					zip->c = spec;
				}
				else
				{
					ban = zip->b;
					shutup = zip->s;
					spec = zip->c;
				}

				memcpy(zip->creator, p->name, 20);
				zip->creator[13] = '\0';
				zip->access = p->access;

				// Return OK-string
				String s;
				s += "Successfully zipped " + String(c->final) + ".  ";
				if (ban)	s += "+ban ";
				if (shutup)	s += "+silence ";
				if (spec)	s += "+spec ";
				s += " Timer: ";
				s += zip->minutes;
				s += "  Access level: ";
				s += zip->access;

				sendPrivate(p, s);

				// Apply settings to player in the zone
				_listnode <Player> *parse = playerlist->head;

				while (parse)
				{
					Player *pp = parse->item;

					if (CMPSTR(pp->name, c->final))
					{
						CheckZip(pp, alias);

						break;
					}

					parse = parse->next;
				}
			}
			else if (c->check("unzip"))
			{
				zip_info *zip = FindZip(c->final);

				if (zip)
				{
					if (p->access < zip->access)
					{
						sendPrivate(p, "You may not modify a zip made by a higher-level operator.");

						break;
					}

					sendPrivate(p, "Killed requested zip.");

					if (!zip->b)
					{	// Apply settings to player in the zone
						_listnode <Player> *parse = playerlist->head;

						while (parse)
						{
							Player *pp = parse->item;

							if (CMPSTR(pp->name, c->final))
							{
								String s = "*warn Zip lifted.";

								// Make sure that the player has the proper status
								ZipRequest *rq = new ZipRequest;
								rq->p = pp;
								rq->c = false;
								rq->s = false;
								rq->c_ok = !zip->c;
								rq->s_ok = !zip->s;
								zip_requests.append(rq);

								if (zip->c)
								{
									s += " You may now enter the arena.";
									sendPrivate(pp, "*spec");
								}

								if (zip->s)
								{
									s += " You may now speak.";
									sendPrivate(pp, "*shutup");
								}

								sendPrivate(pp, s);

								break;
							}

							parse = parse->next;
						}
					}

					KillZip(zip);
				}
				else
				{
					sendPrivate(p, "Unable to find a zipped player by that name.");
				}
			}
			else if (c->check("listzip"))
			{
				bool ban	= (c->getParam('b') != NULL);
				bool shutup	= (c->getParam('s') != NULL);
				bool spec	= (c->getParam('c') != NULL);

				if (ban) { shutup = false; spec = false; }

				_listnode <zip_info> *parse = ZipList.head;
				int count = 0;

				while (parse)
				{
					zip_info *zip = parse->item;
					parse = parse->next;

					if (ban		&& (!zip->b))	continue;
					if (shutup	&& (!zip->s))	continue;
					if (spec	&& (!zip->c))	continue;

					if (count == 0)
					{
						sendPrivate(p, "|Name                    Tags                Minutes   Creator:Level|");
						sendPrivate(p, "|-------------------------------------------------------------------|");
					}

					alias_name *alias = zip->alias;

					String name = alias->name;
					String flag;
					if (zip->b)	flag += "+ban ";
					if (zip->s)	flag += "+silence ";
					if (zip->c)	flag += "+spec ";
					String remain = zip->minutes - (ZipElapsed(zip) / 60);
					String access = zip->creator;
					access += ":";
					if (zip->access > 9)
						access += "^";
					else
						access += zip->access;

					String s; // will be 69 characters long, for 800x600 res
					s += "|";
					s += name.lfill(23);
					s += " ";
					s += flag.rfill(20);
					s += remain.lfill(7);
					s += " ";
					s += access.lfill(15);
					s += "|";
					sendPrivate(p, s);
					++count;
				}

				if (count == 0)
				{
					sendPrivate(p, "No zipped players (matching your query).");
				}
			}
			else if (c->check("alias"))
			{
				alias_name *alias = FindSnapshot(c->final);

				bool approx_ip = (c->getParam('a') != NULL);

				if (!alias)
				{
					sendPrivate(p, "Have no record for them.");
					break;
				}

				bool found = false;
				int count = 0;
				String s = "Alias";
				_listnode <alias_name> *parse = AliasList.head;

loop_head:

				while (parse)
				{
					alias_name *palias = parse->item;
					parse = parse->next;

					if (count == 10)
					{
						sendPrivate(p, s);
						s = "Alias";
						count = 0;
					}

					for (int i = 0; i < MEMORY_LENGTH; ++i)
					{
						alias_data *pdata = palias->data[i];
						if (!pdata) break;

						for (int j = 0; j < MEMORY_LENGTH; ++j)
						{
							alias_data *data = alias->data[j];
							if (!data) break;

							if ((data->id == pdata->id) && (pdata->id != 0))
							{
								found = true;
								s += " :";
								s += palias->name;
								s += ":ID";
								++count;

								goto loop_head;
							}
							else if (data->ip == pdata->ip)
							{
								found = true;
								s += " :";
								s += palias->name;
								s += ":IP";
								++count;

								goto loop_head;
							}
							else if (approx_ip)
							{
								if ((data->ip & 0x00ffffff) == (pdata->ip & 0x00ffffff))
								{
									found = true;
									s += " :";
									s += palias->name;
									s += ":~IP";
									++count;

									goto loop_head;
								}
								else if ((data->ip & 0x0000ffff) == (pdata->ip & 0x0000ffff))
								{
									found = true;
									s += " :";
									s += palias->name;
									s += ":~~IP";
									++count;

									goto loop_head;
								}
							}
						}
					}

					if (alias->timediff != 0)
					{
						Sint32 timediff = alias->timediff - palias->timediff;
						if (timediff < 0) timediff = -timediff;

						if (timediff <= 150)
						{
							found = true;
							s += " :";
							s += palias->name;
							s += "(";
							s += timediff;
							s += "; ";
							s += Sint32(alias->seen - palias->seen) / 60;
							s += " min. apart):";
							++count;

							goto loop_head;
						}
					}
				}

				if (!found) sendPrivate(p, "No matches.");
				else if (count != 0) sendPrivate(p, s);
			}
		}
	case OP_Limited:
		{	// Limited-level commands
			if (c->check("status"))
			{
				String s;
				s += "Recorded ";
				s += AliasList.total;
				s += " aliases total. Waiting for ";
				s += queue.total;
				s += " aliases to resolve, ";
				s += zip_requests.total;
				s += " zips to resolve.";

				sendPrivate(p, s);
			}
		}
	case OP_Player:
		{	// Player-level commands
			if (c->check("about"))
			{
				sendPrivate(p, "I record player information used for linking together player aliases");
			}
			else if (c->check("seen"))
			{
				sendPrivate(p, GetLastSeen(c->final));
			}
		}
	}
}

void botInfo::gotRemoteHelp(char *p, Command *c, Operator_Level l)
{
	// List commands

	if (!*c->final)
	{
		switch (l)
		{
		case OP_Duke:
		case OP_Baron:
		case OP_King:
		case OP_Emperor:
		case OP_RockStar:
		case OP_Q:
		case OP_God:
		case OP_Owner:
			{	// Owner-level commands
			}
		case OP_SysOp:
			{	// SysOp-level commands
			}
		case OP_SuperModerator:
			{	// SuperModerator-level commands
			}
		case OP_Moderator:
			{	// Moderator-level commands
			}
		case OP_Limited:
			{	// Limited-level commands
			}
		case OP_Player:
			{	// Player-level commands
			}
		}

		return;
	}

	// Specific command help

	switch (l)
	{
	case OP_Duke:
	case OP_Baron:
	case OP_King:
	case OP_Emperor:
	case OP_RockStar:
	case OP_Q:
	case OP_God:
	case OP_Owner:
		{	// Owner-level commands
		}
	case OP_SysOp:
		{	// SysOp-level commands
		}
	case OP_SuperModerator:
		{	// SuperModerator-level commands
		}
	case OP_Moderator:
		{	// Moderator-level commands
		}
	case OP_Limited:
		{	// Limited-level commands
		}
	case OP_Player:
		{	// Player-level commands
		}
	}
}

void botInfo::gotRemote(char *p, Command *c, Operator_Level l)
{
	if (!c) return;

	switch (l)
	{
	case OP_Duke:
	case OP_Baron:
	case OP_King:
	case OP_Emperor:
	case OP_RockStar:
	case OP_Q:
	case OP_God:
	case OP_Owner:
		{	// Owner-level commands
		}
	case OP_SysOp:
		{	// SysOp-level commands
		}
	case OP_SuperModerator:
		{	// SuperModerator-level commands
		}
	case OP_Moderator:
		{	// Moderator-level commands
		}
	case OP_Limited:
		{	// Limited-level commands
		}
	case OP_Player:
		{	// Player-level commands
		}
	}
}
